@@ -1,12 +1,232 @@
+import logging
 from decimal import Decimal, ROUND_HALF_UP
-from typing import Tuple, Union
+from typing import Tuple, Union, List, Dict
 import datetime
+import requests
+import json
 
 from st_common_data.utils.common import (
     touch_db_with_dict_response, touch_db,
-    get_current_datetime,
+    get_current_datetime, is_working_day, get_previous_workday,
 )
 
+logger = logging.getLogger(__name__)
+
+
+# --------------------- Queries to Datum API: ---------------------
+def datum_api_get_request(url: str,
+                          service_auth0_token: str,
+                          params: dict = None,
+                          body_params: dict = None):
+    """
+    Return:
+     - json response from Datum API if success
+     - raise Exception if error
+
+    Args:
+        url (str): url of Datum API
+        service_auth0_token (str): auth0 token of service
+        params (dict): query params
+        body_params (dict): body params
+    """
+
+    api_response = requests.request(
+        method='get',
+        url=url,
+        headers={'Authorization': f'Bearer {service_auth0_token}'},
+        params=params,
+        json=body_params
+    )
+
+    if api_response.status_code == 200:
+        return api_response.json()
+    else:
+        err_msg = f'Unable to make Datum API request: {api_response.text}'
+        logger.error(err_msg)
+        raise Exception(err_msg)
+
+
+def api_get_last_n_candles_by_ticker(datum_api_url: str,
+                                     service_auth0_token: str,
+                                     ticker: str,
+                                     n_candles: int,
+                                     interval: int = 5,
+                                     unix_timestamp: int = None,
+                                     ) -> List[dict]:
+    params = {
+        'ticker': ticker,
+        'interval': interval,
+        'n_candles': n_candles,
+        'unix_timestamp': unix_timestamp
+    }
+
+    list_of_dicts = datum_api_get_request(
+        url=f'{datum_api_url}/intraday/last_n_candles',
+        service_auth0_token=service_auth0_token,
+        params=params
+    )
+
+    return list_of_dicts
+
+
+def api_get_reports(datum_api_url: str,
+                    move_date_str: str,
+                    service_auth0_token: str) -> List[str]:
+    """Return list of tickers which have report event as of move_date_str"""
+    list_of_dicts = datum_api_get_request(
+        url=f'{datum_api_url}/reports',
+        service_auth0_token=service_auth0_token,
+        params={'move_date_str': move_date_str}
+    )
+
+    tickers_list = [row['ticker'] for row in list_of_dicts]
+    return tickers_list
+
+
+def api_get_splits(datum_api_url: str,
+                   review_date_str: Union[datetime.date, str],
+                   service_auth0_token: str
+                   ) -> dict:
+    """Return dict of {str: decimal}"""
+    list_of_dicts = datum_api_get_request(
+        url=f'{datum_api_url}/dvd/splits',
+        service_auth0_token=service_auth0_token,
+        params={'date': review_date_str}
+    )
+
+    return {
+        row['ticker']: Decimal(row['amount']).quantize(
+            Decimal('0.01'),
+            rounding=ROUND_HALF_UP
+        ) for row in list_of_dicts
+    }
+
+
+def api_get_dividends(datum_api_url: str,
+                      review_date_str: Union[datetime.date, str],
+                      service_auth0_token: str,
+                      dvd_type: str = 'cash'
+                      ) -> dict:
+    """Return dict of {str: decimal}"""
+
+    if dvd_type == 'cash':
+        url_end = 'dvd/cash_dividends'
+    elif dvd_type == 'stock':
+        url_end = 'dvd/stock_dividends'
+    else:
+        raise Exception(f'Unknown dvd_type: {dvd_type}')
+
+    list_of_dicts = datum_api_get_request(
+        url=f'{datum_api_url}/{url_end}',
+        service_auth0_token=service_auth0_token,
+        params={'date': review_date_str}
+    )
+
+    return {
+        row['ticker']: Decimal(row['amount']).quantize(
+            Decimal('0.01'),
+            rounding=ROUND_HALF_UP
+        ) for row in list_of_dicts
+    }
+
+
+def api_get_adv(datum_api_url: str,
+                service_auth0_token: str,
+                date_as_of: Union[str, datetime.date] = None) -> dict:
+    """Return dict of {str: decimal}"""
+    if not date_as_of:
+        date_as_of = get_previous_workday(get_current_datetime().date())
+    elif not is_working_day(date_as_of):
+        date_as_of = get_previous_workday(date_as_of)
+
+    list_of_dicts = datum_api_get_request(
+        url=f'{datum_api_url}/calculations/avg_daily_value_traded_3m',
+        service_auth0_token=service_auth0_token,
+        params={'date_as_of_str': date_as_of}
+    )
+
+    return {row['ticker']: row['value'] for row in list_of_dicts}
+
+
+def api_get_close_price(datum_api_url: str,
+                        service_auth0_token: str,
+                        date: Union[str, datetime.date] = None,
+                        ticker: str = None) -> dict:
+    if not date:
+        date = get_current_datetime().date()
+
+    params = {'start_date': date, 'end_date': date, 'chart_type': 'ohlc'}
+    if ticker:
+        params['ticker'] = ticker
+
+    list_of_dicts = datum_api_get_request(
+        url=f'{datum_api_url}/daily',
+        service_auth0_token=service_auth0_token,
+        params=params
+    )
+
+    if ticker:
+        return {ticker: list_of_dicts[0]['c']}
+    else:
+        return {row['ticker']: row['c'] for row in list_of_dicts}
+
+
+def api_get_tickers_sector(datum_api_url: str,
+                           service_auth0_token: str,
+                           tickers: List[str]) -> dict:
+    list_of_dicts = datum_api_get_request(
+        url=f'{datum_api_url}/bics_sectors',
+        service_auth0_token=service_auth0_token,
+        body_params={'tickers': tickers}
+    )
+
+    return {row['ticker']: row['lvl3'] for row in list_of_dicts}
+
+
+def api_get_avg_pre_mh_vol(datum_api_url: str,
+                           service_auth0_token: str,
+                           date: datetime.date = None) -> dict:
+    if not date:
+        date = get_current_datetime().date()
+
+    list_of_dicts = datum_api_get_request(
+        url=f'{datum_api_url}/calculations/avg_premarket_volume_90d',
+        service_auth0_token=service_auth0_token,
+        params={'date_as_of': date}
+    )
+    return {row['ticker']: row['value'] for row in list_of_dicts}
+
+
+def api_get_country_list(datum_api_url: str,
+                         service_auth0_token: str,) -> List[str]:
+    tickers_data = datum_api_get_request(
+        url=f'{datum_api_url}/tickers',
+        service_auth0_token=service_auth0_token,
+        params={}
+    )
+    unique_countries = set([row['country'] for row in tickers_data])
+    return list(unique_countries)
+
+
+def api_get_atr(datum_api_url: str,
+                service_auth0_token: str,
+                date: datetime.date = None) -> Dict[str, float]:
+    if not date:
+        date = get_previous_workday(get_current_datetime().date())
+
+    list_of_dicts = datum_api_get_request(
+        url=f'{datum_api_url}/calculations/volatility_20d',
+        service_auth0_token=service_auth0_token,
+        params={'date_as_of_str': date}
+    )
+
+    return {row['ticker']: row['value'] for row in list_of_dicts}
+
+
+# --------------------- End of Queries to Datum API ---------------------
+# 
+# --------------------- Queries to Datum database: ---------------------
+
 
 def get_datum_data_by_ticker(db_creds, ticker, review_date):
     """Return data as of previous work day"""
@@ -44,6 +264,7 @@ def get_datum_data_by_ticker(db_creds, ticker, review_date):
     return datum_data
 
 
+# Will be deprecated (when use Reporting System instead of MSW.DR)
 def premarket_datum_is_ready(db_creds):
     """Check if current Premarket is ready in Datum DB"""
     is_ready = False
@@ -97,6 +318,7 @@ def get_tickers_sector(db_creds, ticker_names_tuple: Tuple[str]):
     return response_dict
 
 
+# Frontend can make this request to Datum API by itself (in MSW: api/trader/sector_list/)
 def get_sector_list(db_creds, level=3):
     response = touch_db(
         """
@@ -115,6 +337,7 @@ def get_sector_list(db_creds, level=3):
     return sector_list
 
 
+# Frontend can make this request to Datum API by itself (in MSW: api/trader/country_list/)
 def get_country_list(db_creds):
     response = touch_db(
         """
@@ -207,7 +430,7 @@ def get_reports(db_creds, review_date_str: Union[datetime.date, str]):
     reports = touch_db(
         f"""
         SELECT ticker_by_esignal, announcement_date, announcement_time
-        FROM tickers_by_company t 
+        FROM tickers_by_company t
         JOIN earnings_date_history edh ON edh.id_company = t.id_company
         JOIN exchange e ON e.id = t.id_exchange
         WHERE e.id_country = (SELECT id FROM country WHERE name = 'UNITED STATES')
@@ -263,7 +486,7 @@ def get_stock_dividends(db_creds, review_date_str: Union[datetime.date, str]):
           SELECT ticker_by_esignal, amount, ex_date
           FROM tickers_by_company t
           LEFT JOIN dvd ON t.id = dvd.id_ticker     
-          WHERE ex_date = '{review_date_str}'  
+          WHERE ex_date = '{review_date_str}'
           AND id_dvd_type = 46;
         """,
         dbp=db_creds
@@ -284,6 +507,7 @@ def get_stock_dividends(db_creds, review_date_str: Union[datetime.date, str]):
     return stock_dividends_dict
 
 
+# Not used anymore
 def get_tickers_gap(db_creds, ticker_names_tuple: Tuple[str], tuple_of_str_dates: Tuple[str]):
     """Response format:
     {'AAPL': {datetime.date(2021, 1, 7): Decimal('1.39')
@@ -385,20 +609,6 @@ def get_average_daily_volume(db_creds, ticker_names_tuple: Tuple[str], effective
     return response_dict
 
 
-def get_close_price(db_creds, date, ticker):
-
-    query = f"""
-                SELECT ticker_by_esignal as ticker, date, clo as close_price
-                FROM tickers_by_company t
-                JOIN opg_and_clo o ON o.id_ticker = t.id
-                JOIN exchange e ON e.id = t.id_exchange
-                JOIN country c ON c.id = e.id_country AND c.name = 'UNITED STATES'
-                WHERE date = ('%s') AND active AND ticker_by_esignal = ('%s')
-            """ % (date, ticker)
-
-    return touch_db_with_dict_response(query=query, dbp=db_creds)
-
-
 # Tier System:
 def get_adv(db_creds, date=None):
     if not date:
@@ -433,6 +643,19 @@ def get_high_low(db_creds, date=None):
     return touch_db_with_dict_response(query=query, dbp=db_creds)
 
 
+def get_close_price(db_creds, date, ticker):
+    query = f"""
+                SELECT ticker_by_esignal as ticker, date, clo as close_price
+                FROM tickers_by_company t
+                JOIN opg_and_clo o ON o.id_ticker = t.id
+                JOIN exchange e ON e.id = t.id_exchange
+                JOIN country c ON c.id = e.id_country AND c.name = 'UNITED STATES'
+                WHERE date = ('%s') AND active AND ticker_by_esignal = ('%s')
+            """ % (date, ticker)
+
+    return touch_db_with_dict_response(query=query, dbp=db_creds)
+
+
 def get_close_price_as_dict(db_creds, date=None):
     if not date:
         date = get_current_datetime().date()
@@ -464,3 +687,5 @@ def get_avg_pre_mh_vol(db_creds, date=None):
             """ % (date, date)
 
     return touch_db_with_dict_response(query=query, dbp=db_creds)
+
+# --------------------- End of Queries to Datum database ---------------------
